#ifndef _CONFIG_FILE_H_

#define _CONFIG_FILE_H_

#include <stdio.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <assert.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include "utility.h"
#include "config.h"
#include "chunkyircd.h"
#include "server.h"
#include "modules.h"

#ifdef USE_OPENSSL
#include "enpress.h"
#endif

#ifdef HAVE_GETOPT_H
#include <getopt.h>
#endif

/* related to getopt() */
extern char *optarg;
extern int optind, opterr, optopt;

#define TLINES_MAX	8	/* Size of tline array */
#define MAX_OPS 	16	/* Size of oline array */
#define MOTD_LINES_MAX 	32	/* Size of motd array */
#define OP_NAME_LEN 	16	/* Size of ircop's name field */
#define OP_PASS_LEN 	16	/* Size of ircop's password field */
#define OP_HOST_LEN 	64	/* Size of ircop's host field */
#define PASS_LENGTH 	32	/* Size of password length */
#define USERMODES 	"iwsort"	/* User Modes */
#define CHANMODES 	"ntovblmkhsrRT"	/* Channel modes */
#define VERSION_TAG 	"2.8/monkeyircd-"	/* Initial Version Tag */

/* Runtime vars describing server state */
struct server_config_globals {
    /* Server Info */
    char server_name[SERVNAME_MAX + 1];
    char server_info[SERVINFO_MAX + 1];
    char conf_file[200];
    char load_date[40];
    char *pass;
    struct irc_op **olines;
    int mass_kicks;
    char **motd;
    int max_chancount;
    int debug;
    int foreground;
    pid_t pid;
	int max_fd_buffer_count;

    /* Services config */
    char services_name[SERVNAME_MAX + 1];
    char services_nickserv[NICK_LENGTH + 1];

    /* Server SSL Config */
#ifdef USE_OPENSSL
    char ssl_cert_file[200];
    char ssl_key_file[200];
    SSL_CTX *ssl_ctx;
#endif
};

struct server_state_globals {
    int ops;
    int services;
    long int channels;
    long int temp_count;
};

/* Variables with server state in them */
extern struct server_config_globals server_config;
extern struct server_state_globals server_state;

/* Irc op structure in memory */
struct irc_op {
    char name[OP_NAME_LEN];
    char pass[OP_PASS_LEN];
    char hostmask[OP_HOST_LEN];
};

/* Cline, array is static to the config file routines */
struct cline {
    char serv_name[SERVNAME_MAX + 1];
    char host[HOST_LENGTH + 1];
    int port;
    char password[PASS_LENGTH + 1];
    unsigned char key[17];
    time_t holdoff;
};

struct nline {
    char serv_name[SERVNAME_MAX + 1];
    char host[HOST_LENGTH + 1];
    char password[PASS_LENGTH + 1];
    unsigned char key[17];
};

/* tline = A trusted IP subnet, for +t mode allocation */
struct tline {
    struct in_addr base;
    struct in_addr netmask;
};


struct config_node {
    const char *name;
    void (*worker) (char *, char *, struct bind_info[]);
};

/* Config file parsing functions */
int parse_config_file(char *, struct bind_info[]);
/* Parse command line */
int parse_cmdline(int, char *[]);
/* CLINE/NLINE Stuff */
struct cline *get_cline(char *);
struct nline *get_nline(char *);
int check_tlines(struct in_addr *);
/* re-read config file */
void rehash(int);

#endif
