/* This file is part of Chunky Monkey IRCD 
 *
 * Chunky Monkey IRCD is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Chunky Monkey IRCD is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Chunky Monkey IRCD; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * $Id: cmd_ping.c,v 1.7.4.17 2003/01/19 12:17:19 lloydy Exp $
 *
 */
#include "modules.h"

CMIRCD_MODULE(1.02, CMD_CLIENT | CMD_SERVER);

/* respond to a ping */
int cmd_ping(ClientInfo c_ptr, ServerInfo s_ptr, Cmd cmd)
{
	char buffer[BUF_SIZE];
	char *target = NULL;
	char *origin = NULL;
	ServerInfo my_serv = NULL;

	my_serv = get_my_server();

	if (!check_params(cmd, 1) && !cmd->text) {
		/* Not enough params */
		if (c_ptr) {
			/* return error */
			send_numeric(c_ptr, my_serv, 409,
				     ":No origin specified");
		}
		return -1;
	}

	/* One of these is non-null */
	if (cmd->params[0]) {
		origin = cmd->params[0];
	} else {
		origin = cmd->text;
	}

	if (cmd->params[1]) {
		target = cmd->params[1];
	}

	if (target == NULL || lc_strncmp(my_serv->name, target, -1) == 0) {
		ircd_sprintf(buffer, BUF_SIZE, "PONG %s %s",
			     my_serv->name, origin);
		if (c_ptr) {
			send_origin_client(c_ptr, my_serv->name, buffer);
		}
		if (s_ptr) {
			send_origin_server(s_ptr, my_serv->name, buffer);
		}
	} else {
		ServerInfo targ_serv = NULL;
		char *fw_origin = NULL;

		/* Can't think of a time when you'd send a ping to a client */
		targ_serv = server_locate(target);

		if (cmd->origin) {
			fw_origin = cmd->origin;
		} else {
			if (c_ptr) {
				fw_origin = c_ptr->nick;
			} else {
				return -1;
			}
		}

		if (targ_serv) {
			ircd_sprintf(buffer, BUF_SIZE, "PONG %s %s",
				     origin, target);
			send_origin_server(targ_serv, fw_origin, buffer);
		} else {
			if (c_ptr) {
				ircd_sprintf(buffer, BUF_SIZE,
					     "%s :No such server", target);
				send_numeric(c_ptr, my_serv, 402, buffer);
			}
		}


	}

	return 1;
}
